using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Diagnostics;

namespace Sequence
{
    public partial class SequenceForm : Form
    {
        /// <summary>
        /// Useful variables
        /// </summary>
        private CheckBox[] _channelSelectionCheckBox;
        private ComboBox[] _channelGainComboBox;
        private ComboBox[] _channelConnectionComboBox;
        private ComboBox[] _channelCouplingComboBox;
        private CheckBox[] _channelIcpCheckBox;
        private double[] _availableFrequencies;
        
        /// <summary>
        /// Data client to our system
        /// </summary>
        private DataClient _client;

        /// <summary>
        /// Store the activated channels (current acquisition)
        /// </summary>
        private bool[] _channelActivated;

        /// <summary>
        /// Sequence size
        /// </summary>
        private int sequenceSize;

        /// <summary>
        /// Store the values of the channels and additional information (timestamp, sequence number...)
        /// </summary>
        private double[] _values;
        private ulong[] _addValues;

        /// <summary> 
        /// The reader and the writer 
        /// </summary> 
        private BinaryWriter _writer;
        private BinaryReader _reader;
        
        /// <summary>
        /// Store the gain of the channels (current acquisition)
        /// </summary>
        private int[] _gain;

        /// <sumary>
        /// Store the dataFormat
        /// </sumary>
        private ulong dataFormat;
        private bool  stop;

        /// <summary>
        /// Store the number of data packets received
        /// </summary>
        private int packetCount;

        /// <summary>
        /// Save data to a file
        /// </summary>
        private bool saveData;

        /// <summary>
        /// Store the number of sequence contained in one packet
        /// </summary>
        private ulong sequenceBeforeTransfer;

        /// <summary>
        /// Constructor
        /// </summary>
        public SequenceForm()
        {
            InitializeComponent();

            // Create the useful variables
            this._channelSelectionCheckBox = new CheckBox[8];
            this._channelSelectionCheckBox[0] = this.channelSelectionCheckBox0;
            this._channelSelectionCheckBox[1] = this.channelSelectionCheckBox1;
            this._channelSelectionCheckBox[2] = this.channelSelectionCheckBox2;
            this._channelSelectionCheckBox[3] = this.channelSelectionCheckBox3;
            this._channelSelectionCheckBox[4] = this.channelSelectionCheckBox4;
            this._channelSelectionCheckBox[5] = this.channelSelectionCheckBox5;
            this._channelSelectionCheckBox[6] = this.channelSelectionCheckBox6;
            this._channelSelectionCheckBox[7] = this.channelSelectionCheckBox7;
            this._channelGainComboBox = new ComboBox[8];
            this._channelGainComboBox[0] = this.gainComboBox0;
            this._channelGainComboBox[1] = this.gainComboBox1;
            this._channelGainComboBox[2] = this.gainComboBox2;
            this._channelGainComboBox[3] = this.gainComboBox3;
            this._channelGainComboBox[4] = this.gainComboBox4;
            this._channelGainComboBox[5] = this.gainComboBox5;
            this._channelGainComboBox[6] = this.gainComboBox6;
            this._channelGainComboBox[7] = this.gainComboBox7;
            this._channelConnectionComboBox = new ComboBox[8];
            this._channelConnectionComboBox[0] = this.connectionComboBox0;
            this._channelConnectionComboBox[1] = this.connectionComboBox1;
            this._channelConnectionComboBox[2] = this.connectionComboBox2;
            this._channelConnectionComboBox[3] = this.connectionComboBox3;
            this._channelConnectionComboBox[4] = this.connectionComboBox4;
            this._channelConnectionComboBox[5] = this.connectionComboBox5;
            this._channelConnectionComboBox[6] = this.connectionComboBox6;
            this._channelConnectionComboBox[7] = this.connectionComboBox7;
            this._channelCouplingComboBox = new ComboBox[8];
            this._channelCouplingComboBox[0] = this.couplingComboBox0;
            this._channelCouplingComboBox[1] = this.couplingComboBox1;
            this._channelCouplingComboBox[2] = this.couplingComboBox2;
            this._channelCouplingComboBox[3] = this.couplingComboBox3;
            this._channelCouplingComboBox[4] = this.couplingComboBox4;
            this._channelCouplingComboBox[5] = this.couplingComboBox5;
            this._channelCouplingComboBox[6] = this.couplingComboBox6;
            this._channelCouplingComboBox[7] = this.couplingComboBox7;
            this._channelIcpCheckBox = new CheckBox[8];
            this._channelIcpCheckBox[0] = this.icpCheckBox0;
            this._channelIcpCheckBox[1] = this.icpCheckBox1;
            this._channelIcpCheckBox[2] = this.icpCheckBox2;
            this._channelIcpCheckBox[3] = this.icpCheckBox3;
            this._channelIcpCheckBox[4] = this.icpCheckBox4;
            this._channelIcpCheckBox[5] = this.icpCheckBox5;
            this._channelIcpCheckBox[6] = this.icpCheckBox6;
            this._channelIcpCheckBox[7] = this.icpCheckBox7;

            this._availableFrequencies = new double[] {
                1000.0, 1280.0, 1562.50, 1600.0, 1666.67, 2000.0, 2500.0, 3125.0, 3200.0, 3333.33, 4000.0, 5000.0,
                6250.0, 6400.0, 6666.67, 8000.0, 10000.0, 12500.0, 12800.0, 13333.33, 16000.0, 16666.67,
                20000.0, 25000.0, 32000.0, 33333.33, 40000.0, 50000.0, 64000.0, 66666.67, 80000.0,
                100000.0, 128000.0 };

            // Initialize class members
            this._channelActivated = new bool[8];
            this._addValues = new ulong[4];
            this._values = new double[8];
            this._gain = new int[8];
            this.packetCount = 0;
            this.saveData = false;
            this.sequenceSize = 0;
            this.stop = false;
        }

        /// <summary>
        /// Occurs when the sample is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SequenceForm_Load(object sender, EventArgs e)
        {
            // Load ip address
            this.ipTextBox0.Text = Properties.Settings.Default.ip0;
            this.ipTextBox1.Text = Properties.Settings.Default.ip1;
            this.ipTextBox2.Text = Properties.Settings.Default.ip2;
            this.ipTextBox3.Text = Properties.Settings.Default.ip3;

            // Initialize the check boxes and combo boxes
            this.allChannelsCheckBox.Checked = true;
            this.allGainComboBox.SelectedIndex = 0;
            this.allConnectionComboBox.SelectedIndex = 0;
            this.allConnectionComboBox.SelectedIndex = 0;
            this.allCouplingComboBox.SelectedIndex = 0;

            this.hardwareTriggerGroupBox.Enabled = false;

            for (int i = 0; i < 8; i++)
            {
                this._channelSelectionCheckBox[i].Checked = true;
                this._channelGainComboBox[i].SelectedIndex = 0;
                this._channelConnectionComboBox[i].SelectedIndex = 0;
                this._channelCouplingComboBox[i].SelectedIndex = 0;
            }
            this.frequencyComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the sample is closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SequenceForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the settings
            Properties.Settings.Default.ip0 = this.ipTextBox0.Text;
            Properties.Settings.Default.ip1 = this.ipTextBox1.Text;
            Properties.Settings.Default.ip2 = this.ipTextBox2.Text;
            Properties.Settings.Default.ip3 = this.ipTextBox3.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Occurs when the user clicks on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            IPAddress ip;
            String ipString = this.ipTextBox0.Text + "." + this.ipTextBox1.Text + "." + this.ipTextBox2.Text + "." + this.ipTextBox3.Text;
            MSXE360xWeb.MSXE360x system = null;

            ulong  channelMask             = 0;
            ulong  couplingSelectionMask   = 0;
            ulong  connectionSelectionMask = 0;
            ulong  icpMask                 = 0;
            double frequency               = 0;
            ulong  numberOfSequence        = 0;
            ulong  triggerMask             = 0;
            ulong  hardwareTriggerEdge     = 0;
            ulong  hardwareTriggerCount    = 0;
            int    syserrno                = 0;

            MSXE360xWeb.unsignedLong8FixedArrayParam channelList = new Sequence.MSXE360xWeb.unsignedLong8FixedArrayParam(); 
            MSXE360xWeb.unsignedLong8FixedArrayParam gainArray = new Sequence.MSXE360xWeb.unsignedLong8FixedArrayParam();

            int returnValue = 0;

            // Check the ip address
            if (IPAddress.TryParse(ipString, out ip) == false)
            {
                MessageBox.Show("Invalid ip address.");
                return;
            }

            // Create the system reference
            system = new Sequence.MSXE360xWeb.MSXE360x();
            system.Url = "http://" + ipString + ":5555";
            system.Timeout = 2000;

            // Create/compute the parameters
            gainArray.ulValue = new ulong[8];
            channelList.ulValue = new ulong[8];

            // Clear textBox screen
            this.displayTextBox.Text = "";
            
            // Reset the number of received packets
            this.packetCount = 0;

            // Configure data format and update sequence size
            dataFormat = 0;

            if (this.timestampDataFrameCheckBox.Checked == true)
            {
                sequenceSize += 2;
                dataFormat |= 1;
            }

            if (this.SequenceCounterDataFrameCheckBox.Checked == true)
            {
                sequenceSize += 1;
                dataFormat |= 4;
            }

            if (this.hardwareTriggerDataFrameCheckBox.Checked == true)
            {
                sequenceSize += 1;
                dataFormat |= 8;
            }

            // Configure each channel and update sequence size
            for (int i = 0; i < 8; i++)
            {
                if (this._channelSelectionCheckBox[i].Checked != true)
                {
                    this._channelActivated[i] = false;
                    gainArray.ulValue[i] = 0;
                    continue;
                }
                
                channelMask |= (ulong)1 << i;
                this._channelActivated[i] = true;
                sequenceSize++;

                if (this._channelCouplingComboBox[i].SelectedIndex == 1)
                    couplingSelectionMask = couplingSelectionMask + (ulong)(1 << i);

                if (this._channelConnectionComboBox[i].SelectedIndex == 1)
                    connectionSelectionMask = connectionSelectionMask + (ulong)(1 << i);

                switch (this._channelGainComboBox[i].SelectedIndex)
                {
                    case 0:
                        gainArray.ulValue[i] = 1;
                        this._gain[i] = 1;
                        break;
                    case 1:
                        gainArray.ulValue[i] = 10;
                        this._gain[i] = 10;
                        break;
                    case 2:
                        gainArray.ulValue[i] = 100;
                        this._gain[i] = 100;
                        break;
                }

                if (this._channelIcpCheckBox[i].Checked)
                    icpMask = icpMask + (ulong)(1 << i);
            }

            // Handle to write data in CSV file
            if (saveDataCheckBox.Checked == true)
            {
                SaveFileDialog dlg = new SaveFileDialog();
                dlg.Filter = "raw files (*raw)|*.raw|All files (*.*)|*.*";

                if (dlg.ShowDialog() != DialogResult.OK)
                {
                    MessageBox.Show("Problem when opening file");
                    return;
                }

                // Free writer if not null
                try
                {
                    if (this._writer != null)
                    {
                        this._writer.Close();
                        this._writer = null;
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Failed to free writer, error : " + ex.ToString());
                    return;
                }

                // Create stream
                FileStream stream = new FileStream(dlg.FileName, FileMode.Create);
                
                // Create writer
                try
                {
                    this._writer = new BinaryWriter(stream);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Failed to create writer, error : " + ex.ToString());
                    return;
                }

                // Add the header -> channelmask, dataFormat followed by gain for corresponding channel
                this._writer.Write(channelMask);
                this._writer.Write(dataFormat);
                for (int i = 0; i < 8; i++)
                {
                    if (((channelMask >> i) & 0x1) == 1)
                        this._writer.Write(this._gain[i]);
                }

                this.saveData = true;
            }

            // Get frequency
            frequency = this._availableFrequencies[this.frequencyComboBox.SelectedIndex];

            // Compute sequence before transfer
            // MSXE send packet of 8192 bytes (2048 32 bits words) if sequenceSize * numberOfSequence is greater than 2048.
            // If it's lower than 2048, MSXE send sequenceSize * numberOfSequence
            // SequenceBeforeTransfer is needed only for this sample to know how long to wait before sending an event in order to display data.
            try
            {
                numberOfSequence = Convert.ToUInt32(this.nbrOfSequenceTextBox.Text);
                if (numberOfSequence == 0 || ((numberOfSequence*(ulong)this.sequenceSize) > 2048))
                    sequenceBeforeTransfer = 2048;
                else
                    sequenceBeforeTransfer = numberOfSequence;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Invalid number of sequence : " + ex.Message);
                return;
            }

            // Configure Triggers
            if (this.hardwareTriggerCheckBox.Checked == true)
                triggerMask |= 1;

            if (this.SoftwareTriggerCheckBox.Checked == true)
                triggerMask |= 2;

            // Hardware trigger configuration
            if (this.hardwareTriggerCheckBox.Checked == true)
            {
                try
                {
                    hardwareTriggerCount = Convert.ToUInt32(this.hardwareTriggerCountTextBox.Text);
                    if (hardwareTriggerCount < 1 || hardwareTriggerCount > 65534)
                    {
                        MessageBox.Show("Invalid hardware trigger count (1 to 65535)");
                        return;
                    }

                }
                catch (Exception ex)
                {
                    MessageBox.Show("Invalid hardware trigger count : " + ex.Message);
                    return;
                }

                if (risingEdgeHardwareTrigger.Checked == true)
                    hardwareTriggerEdge |= 1;

                if (fallingEdgeHardwareTrigger.Checked == true)
                    hardwareTriggerEdge |= 2;
            }

            // be sure no sequence is running
            try
            {
                returnValue = system.AnalogInputStopAndReleaseSequence(out syserrno);
            }
            catch (Exception ex)
            {
                MessageBox.Show("AnalogInputStopAndReleaseSequence error : " + ex.Message);
                if (this._writer != null)
                    this._writer.Close();
                this.displayTimer.Stop();
                return;
            }

            // Start the data client
            this._client = new DataClient(ipString,
                8989,
                (int)(sequenceBeforeTransfer) * sequenceSize * 4, // One packet size
                (int)(numberOfSequence) * sequenceSize * 4, // Total size
                this._writer);
            this._client.packetReceived += new DataClient.PacketReceivedEventHandler(_client_packetReceived);
            this._client.StartListening();
            
            try
            {
                returnValue = system.AnalogInputInitAndStartSequence((ulong)channelMask,
                    numberOfSequence,
                    0,
                    frequency,
                    gainArray,
                    icpMask,
                    triggerMask,
                    0,
                    hardwareTriggerEdge,
                    hardwareTriggerCount,
                    0,
                    dataFormat,
                    couplingSelectionMask,
                    connectionSelectionMask,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    out syserrno);
            }
            catch (Exception ex)
            {
                MessageBox.Show("AnalogInputInitAndStartSequence error : " + ex.Message);
                if (this._writer != null)
                    this._writer.Close();
                this.displayTimer.Stop();
                return;
            }
            // Error checking
            if ((returnValue != 0) || (syserrno != 0))
            {
                MessageBox.Show("AnalogInputInitAndStartSequence error : " + returnValue.ToString() + " syserrno : " + syserrno.ToString());
                return;
            }
            // Free the resources
            system.Dispose();
           
            // Start the timer
            this.sequenceStatusTimer.Start();
            this.displayTimer.Start();
            this.progressBar.Visible = true;            // Init the sequence

            // Configure sample display
            this.channelGroupBox.Enabled = false;
            this.sequenceConfigurationGroupBox.Enabled = false;
            this.triggerGroupBox.Enabled = false;
            this.DataFrameConfigurationGroupBox.Enabled = false;
            this.ipTextBox0.Enabled = false;
            this.ipTextBox1.Enabled = false;
            this.ipTextBox2.Enabled = false;
            this.ipTextBox3.Enabled = false;
            this.stopButton.Enabled = true;
            this.startButton.Enabled = false;
        }

        /// <summary>
        /// Return the volt value given a raw value and its configuration
        /// </summary>
        /// <param name="rawValue">Value measured by the system (24 bits)</param>
        /// <param name="gain">1,10 or 100</param>
        /// <param name="connectionType">0: single-ended, 1: differential</param>
        /// <returns>The value in volt</returns>
        double GetVoltValue(double rawValue, int gain, byte connectionType)
        {
            double maxValue = 0;

            // Get the maximum value
            switch (gain)
            {
                case 1:
                    if (connectionType == 0)
                        maxValue = 10.0;
                    else
                        maxValue = 5.0;
                    break;
                case 10:
                    if (connectionType == 0)
                        maxValue = 1.0;
                    else
                        maxValue = 0.5;
                    break;
                case 100:
                    if (connectionType == 0)
                        maxValue = 0.1;
                    else
                        maxValue = 0.05;
                    break;
            }

            // Shift the value
            rawValue = rawValue - 8388607.0;

            // Return the computed value
            return ((maxValue * rawValue) / 8388607.0);
        }

        /// <summary>
        /// Receive a packet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _client_packetReceived(object sender, DataClientEventArgs e)
        {
            ulong offsetValue = 0;
            // Data are not all displayed, only the last data is displayed so we get the corresponding offset
            int offset = e.dataCount - this.sequenceSize * 4;

            this.packetCount += e.dataCount / (this.sequenceSize * 4);

            if ((this.dataFormat & 0x1) != 0)
            {
                this._addValues[offsetValue++] = (ulong)(e.data[offset + 3] << 24 | e.data[offset + 2] << 16 | e.data[offset + 1] << 8 | e.data[offset]);
                offset += 4;
                this._addValues[offsetValue++] = (ulong)(e.data[offset + 3] << 24 | e.data[offset + 2] << 16 | e.data[offset + 1] << 8 | e.data[offset]);
                offset += 4;
            }

            if ((this.dataFormat & 0x4) != 0)
            {
                this._addValues[offsetValue++] = (ulong)(e.data[offset + 3] << 24 | e.data[offset + 2] << 16 | e.data[offset + 1] << 8 | e.data[offset]);
                offset += 4;
            }

            if ((this.dataFormat & 0x8) != 0)
            {
                this._addValues[offsetValue++] = (ulong)(e.data[offset + 3] << 24 | e.data[offset + 2] << 16 | e.data[offset + 1] << 8 | e.data[offset]);
                offset += 4;
            }

            // Read and convert only the last sequence values (one per channel)
            for (int i = 0; i < 8; i++)
            {
                if (this._channelActivated[i] == true)
                {
                    // Convert value in double and get the corresponding voltage
                    this._values[i] = this.GetVoltValue((double)(e.data[3 + offset] << 24 | e.data[2 + offset] << 16 | e.data[1 + offset] << 8 | e.data[offset]), this._gain[i], 0);
                    offset += 4;
                }
                else
                {
                    this._values[i] = 0.0;
                }
            }
        }

        /// <summary>
        ///  Release sample
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param
        private void releaseSample()
        {

            // Stop the client
            if (this._client != null)
            {
                this._client.Close();
            }
            
            // Reset sequence size
            sequenceSize = 0;
            this.statusLabel.Text = "";

            // Stop the timer
            this.sequenceStatusTimer.Stop();

            // Update the sample display
            this.channelGroupBox.Enabled = true;
            this.sequenceConfigurationGroupBox.Enabled = true;
            this.triggerGroupBox.Enabled = true;
            this.DataFrameConfigurationGroupBox.Enabled = true;
            this.ipTextBox0.Enabled = true;
            this.ipTextBox1.Enabled = true;
            this.ipTextBox2.Enabled = true;
            this.ipTextBox3.Enabled = true;
            this.progressBar.Visible = false;
            this.packetCountToolStripStatusLabel.Text = "";
            this.startButton.Enabled = true;
            this.stopButton.Enabled = false;

            if (saveData == true)
            {
                if (this._writer != null)
                {
                    // Close the binary writer 
                    try
                    {
                        this._writer.Close();
                        this._writer = null;
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Failed to close writer");
                        return;
                    }
                }
            }
        }
        /// <summary>
        /// Occurs when the user clicks on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            IPAddress ip;
            String ipString = this.ipTextBox0.Text + "." + this.ipTextBox1.Text + "." + this.ipTextBox2.Text + "." + this.ipTextBox3.Text;
            MSXE360xWeb.MSXE360x system = null;
            int syserrno = 0;
            int returnValue = 0;

            // Check the ip address
            if (IPAddress.TryParse(ipString, out ip) == false)
            {
                MessageBox.Show("Invalid ip address.");
                return;
            }

            // Create the system reference
            system = new Sequence.MSXE360xWeb.MSXE360x();
            system.Url = "http://" + ipString + ":5555";
            system.Timeout = 2000;

            // Stop
            try
            {
                returnValue = system.AnalogInputStopAndReleaseSequence(out syserrno);
            }
            catch (Exception ex)
            {
                MessageBox.Show("AnalogInputStopAndReleaseSequence error : " + ex.Message);
                return;
            }

            // Error checking
            if ((returnValue != 0) || (syserrno != 0))
            {
                MessageBox.Show("AnalogInputStopAndReleaseSequence error : " + returnValue.ToString() + " syserrno : " + syserrno.ToString());
                return;
            }

            releaseSample();

            this.statusLabel.Text = "Sequence stopped.";
            this.statusLabel.ForeColor = Color.Green;

            // free the resources
            system.Dispose();

            stop = true;
        }

        /// <summary>
        /// Occurs when the timer ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            ulong offset   = 0;
            String outline = "";
         
            if ((this.dataFormat & 0x1) != 0)
            {
                outline = "ts = " + this._addValues[offset].ToString() + " tus = " + this._addValues[offset + 1].ToString("D6") + Environment.NewLine;
                offset += 2;
            }

            if ((this.dataFormat & 0x4) != 0)
            {
                outline += "sequence counter = " + String.Format("{0:0}", this._addValues[offset]) + Environment.NewLine;
                offset += 1;
            }

            if ((this.dataFormat & 0x8) != 0)
            {
                outline += "hardware trigger status = " + this._addValues[offset] + Environment.NewLine;
                offset += 1;
            }

            if (outline != "")
                outline += Environment.NewLine; 

            for (ulong i = 0; i < 8; i++)
            {
                outline += "channel " + i + " = "; 
                if (this._channelActivated[i] == true)
                    outline += String.Format("{0:00.00}", this._values[i]) + " V\t";
                else
                    outline += "-\t";
                if (i == 3)
                    outline += Environment.NewLine + Environment.NewLine;
            }
            this.displayTextBox.Text = outline;

            // Display packet count
            this.packetCountToolStripStatusLabel.Text = "Number of sequences received: " + this.packetCount.ToString();

            if (stop == true)
            {
                this.displayTimer.Stop();

                // Free channels
                for (int i = 0; i < 4; i++)
                    this._addValues[i] = 0;

                for (int i = 0; i < 8; i++)
                    this._values[i] = 0;

                // Reset sequence size and other parameters
                sequenceSize = 0;

                stop = false;
            }
        }

        private void convertRawFileToCsvFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ulong  channelMask = 0;
            ulong  numberOfChannels = 0;
            ulong  numberOfAddData  = 0;
            ulong  dataFormat = 0;

            byte[] read;
            int[]  channelGain = new int[8]; 
            long   bytesRead   = 16;
            
            ulong  linesCount = 0;
            ulong  numberOfLines = 0;

            // File handling
            String fileName = "";
            String initialFileName = "";
            int stringIndex = 0;
            int fileIndex = 0;

            // Get number of lines form
            getNumberOfLines getNumberOfLines = new getNumberOfLines();

            // Dialog form
            OpenFileDialog RawFile = new OpenFileDialog();
            SaveFileDialog CsvFile = new SaveFileDialog();

            // Stream handler
            TextWriter CsvWriter = null;
            FileStream RawStream = null;

            // Try to open Raw file
            try
            {
                RawFile.Title =  "Open RAW file to convert";
                RawFile.Filter = "raw files (*raw)|*.raw|All files (*.*)|*.*";
                if (RawFile.ShowDialog() != DialogResult.OK)
                    return;
                
                RawStream = new FileStream(RawFile.FileName, FileMode.Open);
                this._reader = new BinaryReader(RawStream);
            }
            catch(Exception ex)
            {
                MessageBox.Show("Can't open raw file, file may be open, error : " + ex.ToString());
                return;
            }

            // Try to create CSV file
            try
            {
                CsvFile.Title = "Select CSV file to create";
                CsvFile.Filter = "csv files (*csv)|*.csv|All files (*.*|*.*)";
                if (CsvFile.ShowDialog() != DialogResult.OK)
                {
                    this._reader.Close();
                    RawStream.Close();
                    return;
                }
                CsvWriter = new StreamWriter(CsvFile.FileName);

                // Get string properties
                fileName = CsvFile.FileName;
                initialFileName = CsvFile.FileName;
                stringIndex = fileName.IndexOf(".csv");
            }
            catch(Exception ex)
            {
                MessageBox.Show("Can't create CSV file, error : " + ex.ToString());
                this._reader.Close();
                RawStream.Close();
                return;
            }

            // Read Header -> dataFormat (8 bytes) + number of channels (8 bytes) and gain (4 bytes)
            try
            {
                long test = this._reader.BaseStream.Length;
                int gainCpt = 0;

                // Get channelMask
                channelMask = this._reader.ReadUInt64();
                // Get dataFormat
                dataFormat  = this._reader.ReadUInt64();

                if ((dataFormat & 0x1) != 0)
                {
                    CsvWriter.Write("ts;tus;");
                    numberOfAddData+=2;
                }

                if ((dataFormat & 0x4) != 0)
                {
                    CsvWriter.Write("sequenceNumber;");
                    numberOfAddData++;
                }

                if ((dataFormat & 0x8) != 0)
                {
                    CsvWriter.Write("hardwareTriggerStatus;");
                    numberOfAddData++;
                }
                for (int i = 0; i < 8; i++)
                {
                    if (((channelMask >> i) & 0x1) != 0)
                    {
                        CsvWriter.Write("Channel " + i + "(V);");
                        channelGain[gainCpt++] = this._reader.ReadInt32();
                        bytesRead += 4;
                        numberOfChannels++;
                    }
                }
                CsvWriter.WriteLine("");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error while reading header of file : " + ex.Message);
                this._reader.Close();
                RawStream.Close();
                CsvWriter.Close();
            }

            // Get number of lines 0 -> only 1 file
            getNumberOfLines.ShowDialog(this);
            numberOfLines = getNumberOfLines.numberOfLines;

            // Read RAW file and write on CSV file
            while (bytesRead < this._reader.BaseStream.Length)
            {
                // Increase number of lines
                linesCount++;
                for (ulong i = 0; i < numberOfAddData; i++)
                {
                    // Read value for channel
                    read = this._reader.ReadBytes(4);
                    bytesRead += 4;

                    if (read.Length != 4)
                    {
                        MessageBox.Show("Error when reading values wrong file");
                        this._reader.Close();
                        CsvWriter.Close();
                        return;
                    }
                    ulong value = (ulong)(read[3] << 24 |
                            read[2] << 16 |
                            read[1] << 8 |
                            read[0]);
                    
                    // Write the value in CSV file
                    CsvWriter.Write(value + ";");    
                }
                for (ulong i = 0; i < numberOfChannels; i++)
                {
                    // Read value for channel
                    read = this._reader.ReadBytes(4);
                    bytesRead += 4;

                    if (read.Length != 4)
                    {
                        MessageBox.Show("Error when reading values wrong file");
                        this._reader.Close();
                        CsvWriter.Close();
                        return;
                    }

                    // Compute the value
                    double tmp = (double)(read[3] << 24 |
                        read[2] << 16 |
                        read[1] << 8 |
                        read[0]);

                    // Convert the value in V
                    double value = this.GetVoltValue(tmp, channelGain[i], 0);

                    // Write the value in CSV file
                    CsvWriter.Write(value + ";");
                }
                // New line
                CsvWriter.WriteLine("");

                // Close file and create new one if number of lines is reached
                if (numberOfLines != 0)
                {
                    if ((linesCount % numberOfLines == 0) && (bytesRead < this._reader.BaseStream.Length))
                    {
                        fileIndex++;
                        fileName = initialFileName;
                        CsvWriter.Close();
                        fileName = fileName.Insert(stringIndex, fileIndex.ToString());
                        CsvWriter = new StreamWriter(fileName);
                    }
                }
            }

            this._reader.Close();
            CsvWriter.Close();
        }

        private void allChannelsCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.allChannelsCheckBox.Checked == true)
            {
                channelSelectionCheckBox0.Checked = true;
                channelSelectionCheckBox1.Checked = true;
                channelSelectionCheckBox2.Checked = true;
                channelSelectionCheckBox3.Checked = true;
                channelSelectionCheckBox4.Checked = true;
                channelSelectionCheckBox5.Checked = true;
                channelSelectionCheckBox6.Checked = true;
                channelSelectionCheckBox7.Checked = true;
            }
            else
            {
                channelSelectionCheckBox0.Checked = false;
                channelSelectionCheckBox1.Checked = false;
                channelSelectionCheckBox2.Checked = false;
                channelSelectionCheckBox3.Checked = false;
                channelSelectionCheckBox4.Checked = false;
                channelSelectionCheckBox5.Checked = false;
                channelSelectionCheckBox6.Checked = false;
                channelSelectionCheckBox7.Checked = false;
            }
        }

        private void allICPCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.allICPCheckBox.Checked == true)
            {
                icpCheckBox0.Checked = true;
                icpCheckBox1.Checked = true;
                icpCheckBox2.Checked = true;
                icpCheckBox3.Checked = true;
                icpCheckBox4.Checked = true;
                icpCheckBox5.Checked = true;
                icpCheckBox6.Checked = true;
                icpCheckBox7.Checked = true;
            }
            else
            {
                icpCheckBox0.Checked = false;
                icpCheckBox1.Checked = false;
                icpCheckBox2.Checked = false;
                icpCheckBox3.Checked = false;
                icpCheckBox4.Checked = false;
                icpCheckBox5.Checked = false;
                icpCheckBox6.Checked = false;
                icpCheckBox7.Checked = false;
            }
        }

        private void allGainComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            gainComboBox0.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox1.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox2.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox3.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox4.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox5.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox6.SelectedIndex = allGainComboBox.SelectedIndex;
            gainComboBox7.SelectedIndex = allGainComboBox.SelectedIndex;
        }

        private void allConnectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            connectionComboBox0.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox1.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox2.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox3.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox4.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox5.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox6.SelectedIndex = allConnectionComboBox.SelectedIndex;
            connectionComboBox7.SelectedIndex = allConnectionComboBox.SelectedIndex;
        }

        private void allCouplingComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            couplingComboBox0.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox1.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox2.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox3.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox4.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox5.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox6.SelectedIndex = allCouplingComboBox.SelectedIndex;
            couplingComboBox7.SelectedIndex = allCouplingComboBox.SelectedIndex;
        }

        private void sequenceStatusTimer_Tick(object sender, EventArgs e)
        {
            ulong status = 0;
            IPAddress ip;
            String ipString = this.ipTextBox0.Text + "." + this.ipTextBox1.Text + "." + this.ipTextBox2.Text + "." + this.ipTextBox3.Text;
            MSXE360xWeb.MSXE360x system = null;
            MSXE360xWeb.DefaultResponse retval = new MSXE360xWeb.DefaultResponse();

            // Check the ip address
            if (IPAddress.TryParse(ipString, out ip) == false)
            {
                MessageBox.Show("Invalid ip address.");
                return;
            }

            // Create the system reference
            system = new Sequence.MSXE360xWeb.MSXE360x();
            system.Url = "http://" + ipString + ":5555";
            system.Timeout = 2000;

            try
            {
                retval = system.AnalogInputGetSequenceStatus(out status);
                switch (status)
                {
                    case 0:
                        this.progressBar.Visible = false;
                        this.statusLabel.Text = "No sequence running";
                        this.statusLabel.ForeColor = Color.Green;
                        break;
                    case 1:
                        this.progressBar.Visible = true;
                        this.statusLabel.Text = "Sequence running";
                        this.statusLabel.ForeColor = Color.Red;
                        break;
                    case 2:
                        stop = true;
                        // free the resources
                        this.releaseSample();
                        system.Dispose();
                        this.progressBar.Visible = false;
                        this.statusLabel.Text = "Sequence end";
                        this.statusLabel.ForeColor = Color.Blue;
                        break;
                    case 3:
                        this.progressBar.Visible = false;
                        this.statusLabel.Text = "sequence wait trigger";
                        this.statusLabel.ForeColor = Color.Orange;
                        break;
                    case 4:
                        this.progressBar.Visible = false;
                        this.statusLabel.Text = "sequence PLD FIFO overflow";
                        this.statusLabel.ForeColor = Color.Black;
                        break;
                    case 5:
                        this.progressBar.Visible = false;
                        this.statusLabel.Text = "sequence FIFO overflow";
                        this.statusLabel.ForeColor = Color.Black;
                        break;
                }
            }
            catch (Exception ex)
            {
                this.sequenceStatusTimer.Stop();
                // free the resources
                this.releaseSample();
                system.Dispose();
                MessageBox.Show("AnalogInputGetSequenceStatus error : " + ex.Message);
                return;
            }

            // Error checking
            if ((retval.iReturnValue != 0) || (retval.syserrno != 0))
            {
                this.sequenceStatusTimer.Stop();
                this.releaseSample();
                system.Dispose();
                MessageBox.Show("AnalogInputGetSequenceStatus error : " + retval.iReturnValue.ToString() + " syserrno : " + retval.syserrno.ToString());
                return;
            }

        }

        private void hardwareTriggerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.hardwareTriggerCheckBox.Checked == true)
            {
                this.hardwareTriggerGroupBox.Enabled = true;
                this.risingEdgeHardwareTrigger.Checked = true;
                this.fallingEdgeHardwareTrigger.Checked = false;
            }
            else
            {
                this.hardwareTriggerGroupBox.Enabled = false;
                this.fallingEdgeHardwareTrigger.Checked = false;
                this.risingEdgeHardwareTrigger.Checked = false;
            }
        }
    }
}